@@ -79,6 +79,11 @@ def nvcc_version():
 
 def debug_report():
     max_dots = 33
+
+    hip_version = 'unknown'
+    if hasattr(torch.version, 'hip'):
+        hip_version = torch.version.hip
+
     report = [
         ("torch install path",
          torch.__path__),
@@ -86,6 +91,8 @@ def debug_report():
          torch.__version__),
         ("torch cuda version",
          torch.version.cuda),
+        ("torch hip version",
+         hip_version),
         ("nvcc version",
          nvcc_version()),
         ("deepspeed install path",
@@ -94,7 +101,8 @@ def debug_report():
          f"{deepspeed.__version__}, {deepspeed.__git_hash__}, {deepspeed.__git_branch__}"
          ),
         ("deepspeed wheel compiled w.",
-         f"torch {torch_info['version']}, cuda {torch_info['cuda_version']}"),
+         f"torch {torch_info['version']}, cuda {torch_info['cuda_version']}, hip {torch_info['hip_version']}"
+         ),
     ]
     print("DeepSpeed general environment info:")
     for name, value in report:
@@ -107,7 +115,7 @@ def parse_arguments():
         '--hide_operator_status',
         action='store_true',
         help=
-        'Suppress display of installation and compatiblity statuses of DeepSpeed operators. '
+        'Suppress display of installation and compatibility statuses of DeepSpeed operators. '
     )
     parser.add_argument('--hide_errors_and_warnings',
                         action='store_true',
