@@ -364,7 +364,7 @@ class TopKGate(Module):
                  k: int = 1,
                  capacity_factor: float = 1.0,
                  eval_capacity_factor: float = 1.0,
-                 min_capacity: int = 4,
+                 min_capacity: int = 8,
                  noisy_gate_policy: Optional[str] = None,
                  drop_tokens: bool = True,
                  use_rts: bool = True) -> None:
@@ -447,14 +447,16 @@ class MOELayer(Base):
     def __init__(self,
                  gate: Module,
                  experts: Module,
+                 ep_group_name,
+                 ep_size,
                  num_local_experts: int,
-                 group: Optional[Any] = None,
                  use_tutel: bool = False) -> None:
         super().__init__()
         self.gate = gate
         self.experts = experts
-        self.group = group
-        self.world_size = dist.get_world_size(group)
+        self.ep_group = None
+        self.ep_size = ep_size
+        self.ep_group_name = ep_group_name
         self.num_local_experts = num_local_experts
         self.time_falltoall = 0.0
         self.time_salltoall = 0.0
@@ -470,6 +472,9 @@ class MOELayer(Base):
             logger.warning("Tutel optimization requested but not installed. "
                            "Proceeding without Tutel.")
 
+    def _set_ep_group(self, ep_group):
+        self.ep_group = ep_group
+
     def forward(self, *input: Tensor, **kwargs: Any) -> Tensor:
 
         if self.wall_clock_breakdown:
@@ -504,14 +509,14 @@ class MOELayer(Base):
         if self.wall_clock_breakdown:
             self.timers('falltoall').start()
 
-        dispatched_input = _AllToAll.apply(self.group, dispatched_input)
+        dispatched_input = _AllToAll.apply(self.ep_group, dispatched_input)
 
         if self.wall_clock_breakdown:
             self.timers('falltoall').stop()
             self.time_falltoall = self.timers('falltoall').elapsed(reset=False) * 1000
 
         # Re-shape after all-to-all: ecm -> gecm
-        dispatched_input = dispatched_input.reshape(self.world_size,
+        dispatched_input = dispatched_input.reshape(self.ep_size,
                                                     self.num_local_experts,
                                                     -1,
                                                     d_model)
@@ -521,14 +526,14 @@ class MOELayer(Base):
         if self.wall_clock_breakdown:
             self.timers('salltoall').start()
 
-        expert_output = _AllToAll.apply(self.group, expert_output)
+        expert_output = _AllToAll.apply(self.ep_group, expert_output)
 
         if self.wall_clock_breakdown:
             self.timers('salltoall').stop()
             self.time_salltoall = self.timers('salltoall').elapsed(reset=False) * 1000
 
         # Re-shape back: gecm -> ecm
-        expert_output = expert_output.reshape(self.world_size * self.num_local_experts,
+        expert_output = expert_output.reshape(self.ep_size * self.num_local_experts,
                                               -1,
                                               d_model)
 
