@@ -19,14 +19,15 @@ class FusedAdamBuilder(CUDAOpBuilder):
         return ['csrc/adam/fused_adam_frontend.cpp', 'csrc/adam/multi_tensor_adam.cu']
 
     def include_paths(self):
-        return ['csrc/includes']
+        return ['csrc/includes', 'csrc/adam']
 
     def cxx_args(self):
         args = super().cxx_args()
         return args + self.version_dependent_macros()
 
     def nvcc_args(self):
-        return ['-lineinfo',
-                '-O3',
-                '--use_fast_math'
-                ] + self.version_dependent_macros() + self.compute_capability_args()
+        nvcc_flags = ['-O3'] + self.version_dependent_macros()
+        if not self.is_rocm_pytorch():
+            nvcc_flags.extend(['-lineinfo',
+                               '--use_fast_math'] + self.compute_capability_args())
+        return nvcc_flags
