@@ -11,7 +11,7 @@ from torch._utils import _flatten_dense_tensors, _unflatten_dense_tensors
 from deepspeed.runtime.utils import get_global_norm, get_grad_norm, CheckOverflow, get_weight_norm
 from deepspeed.runtime.fp16.loss_scaler import INITIAL_LOSS_SCALE, SCALE_WINDOW, MIN_LOSS_SCALE
 from deepspeed.utils import groups, logger, log_dist
-from deepspeed.runtime.constants import OPTIMIZER_STATE_DICT
+from deepspeed.checkpoint.constants import OPTIMIZER_STATE_DICT
 import torch.distributed as dist
 
 
@@ -141,6 +141,7 @@ class FP16_Optimizer(object):
         """
         Not supporting closure.
         """
+
         # First compute norm for all group so we know if there is overflow
         grads_groups_flat = []
         norm_groups = []
@@ -262,7 +263,7 @@ class FP16_Optimizer(object):
         if self.using_pipeline:
             pg = self.deepspeed.mpu.get_data_parallel_group()
         else:
-            pg = groups.get_data_parallel_group()
+            pg = groups._get_data_parallel_group()
         scaled_norm = all_groups_norm * 1.0 / float(dist.get_world_size(group=pg))
         scaled_norm_tensor = torch.tensor(scaled_norm,
                                           device=self.fp32_groups_flat[i].device,
