@@ -51,10 +51,6 @@ class SDLoaderBase(ABC):
         self.module_key = module_key
         num_ckpt = len(self.ckpt_list)
         idx = mp_rank * num_ckpt // mp_world_size
-
-        logger.info(
-            f'mp_world_size: {mp_world_size}, mp_rank: {mp_rank}, module_key: {module_key}'
-        )
         """ We have multiple cases to handle here for both training and inference:
             1. PipeModule loading mp_rank_*.pt files, is_pipe_parallel=True, module_key is not None
                 a. if no mp_size/pp_size resizing occurs, for both training & inference, loading
@@ -67,7 +63,7 @@ class SDLoaderBase(ABC):
                 a. if no mp_size resizing occurs, for both training & inference, loading
                    the mp_rank related checkpoint directly.
                 b. if has mp_size resizing, only Megatron model inference is supported,
-                   checkpoint file(s) will be merged/splitted according to mp_rank, mp_world_size and
+                   checkpoint file(s) will be merged/split according to mp_rank, mp_world_size and
                    checkpoint file list.
 
             3. Non-PipeModule loading mp_rank_*.pt files, is_pipe_parallel=False
@@ -82,7 +78,7 @@ class SDLoaderBase(ABC):
         merge_count = 1
         if num_ckpt == mp_world_size:
             assert os.path.exists(load_path)
-            logger.info(f'rank: {mp_rank} loading checkpoint: {load_path}')
+            #logger.info(f'rank: {mp_rank} loading checkpoint: {load_path}')
             sd = torch.load(load_path, map_location=lambda storage, loc: storage)
 
             if quantize:
@@ -162,7 +158,7 @@ class SDLoaderBase(ABC):
         return sd
 
     def check_ckpt_list(self):
-        logger.info(f'checkpoint file list: {self.ckpt_list}')
+        #logger.info(f'checkpoint file list: {self.ckpt_list}')
         assert len(self.ckpt_list) > 0
 
         sd = torch.load(self.ckpt_list[0], map_location=lambda storage, loc: storage)
@@ -437,7 +433,7 @@ class MegatronSDLoader(SDLoaderBase):
 
         sd = torch.load(ckpt_file_name, map_location=lambda storage, loc: storage)
 
-        # partail_key is a sub-string of one key in the sd
+        # partial_key is a sub-string of one key in the sd
         def check_key_exist(partial_key, sd):
             keys = sd.keys()
             found = False
